@@ -27,7 +27,6 @@
 #include "optionspage.h"
 #include "constants.h"
 #include "mercurialclient.h"
-#include "mercurialcontrol.h"
 #include "mercurialeditor.h"
 #include "revertdialog.h"
 #include "srcdestdialog.h"
@@ -37,12 +36,12 @@
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/command.h>
+#include <coreplugin/documentmanager.h>
 #include <coreplugin/id.h>
 #include <coreplugin/vcsmanager.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
-#include <coreplugin/documentmanager.h>
 #include <coreplugin/editormanager/editormanager.h>
 
 #include <coreplugin/locator/commandlocator.h>
@@ -55,8 +54,8 @@
 #include <vcsbase/vcsbaseeditor.h>
 #include <vcsbase/vcsbaseconstants.h>
 #include <vcsbase/vcsoutputwindow.h>
+#include <vcsbase/vcscommand.h>
 
-#include <QtPlugin>
 #include <QAction>
 #include <QMenu>
 #include <QDebug>
@@ -71,88 +70,222 @@
 
 using namespace VcsBase;
 using namespace Utils;
+using namespace std::placeholders;
 
 namespace Mercurial {
 namespace Internal {
 
-static const VcsBaseEditorParameters editorParameters[] = {
+class MercurialTopicCache : public Core::IVersionControl::TopicCache
 {
+public:
+    MercurialTopicCache(MercurialClient *client) : m_client(client) {}
+
+protected:
+    QString trackFile(const QString &repository) override
+    {
+        return repository + QLatin1String("/.hg/branch");
+    }
+
+    QString refreshTopic(const QString &repository) override
+    {
+        return m_client->branchQuerySync(repository);
+    }
+
+private:
+    MercurialClient *m_client;
+};
+
+const VcsBaseEditorParameters logEditorParameters {
     LogOutput,
     Constants::FILELOG_ID,
     Constants::FILELOG_DISPLAY_NAME,
-    Constants::LOGAPP},
-
-{   AnnotateOutput,
-    Constants::ANNOTATELOG_ID,
-    Constants::ANNOTATELOG_DISPLAY_NAME,
-    Constants::ANNOTATEAPP},
-
-{   DiffOutput,
-    Constants::DIFFLOG_ID,
-    Constants::DIFFLOG_DISPLAY_NAME,
-    Constants::DIFFAPP}
+    Constants::LOGAPP
 };
 
-static const VcsBaseSubmitEditorParameters submitEditorParameters = {
+const VcsBaseEditorParameters annotateEditorParameters {
+    AnnotateOutput,
+    Constants::ANNOTATELOG_ID,
+    Constants::ANNOTATELOG_DISPLAY_NAME,
+    Constants::ANNOTATEAPP
+};
+
+const VcsBaseEditorParameters diffEditorParameters {
+    DiffOutput,
+    Constants::DIFFLOG_ID,
+    Constants::DIFFLOG_DISPLAY_NAME,
+    Constants::DIFFAPP
+};
+
+const VcsBaseSubmitEditorParameters submitEditorParameters {
     Constants::COMMITMIMETYPE,
     Constants::COMMIT_ID,
     Constants::COMMIT_DISPLAY_NAME,
     VcsBaseSubmitEditorParameters::DiffFiles
 };
 
-MercurialPlugin *MercurialPlugin::m_instance = 0;
-
-MercurialPlugin::MercurialPlugin()
+class MercurialPluginPrivate final : public VcsBase::VcsBasePluginPrivate
 {
-    m_instance = this;
-}
+    Q_DECLARE_TR_FUNCTIONS(Mercurial::Internal::MercurialPlugin)
+
+public:
+    MercurialPluginPrivate();
+
+    // IVersionControl
+    QString displayName() const final;
+    Core::Id id() const final;
+    bool isVcsFileOrDirectory(const Utils::FilePath &fileName) const final;
+
+    bool managesDirectory(const QString &filename, QString *topLevel = nullptr) const final;
+    bool managesFile(const QString &workingDirectory, const QString &fileName) const final;
+    bool isConfigured() const final;
+    bool supportsOperation(Operation operation) const final;
+    bool vcsOpen(const QString &fileName) final;
+    bool vcsAdd(const QString &filename) final;
+    bool vcsDelete(const QString &filename) final;
+    bool vcsMove(const QString &from, const QString &to) final;
+    bool vcsCreateRepository(const QString &directory) final;
+    bool vcsAnnotate(const QString &file, int line) final;
+
+    Core::ShellCommand *createInitialCheckoutCommand(const QString &url,
+                                                     const Utils::FilePath &baseDirectory,
+                                                     const QString &localName,
+                                                     const QStringList &extraArgs) final;
+
+    bool sccManaged(const QString &filename);
+
+    // To be connected to the HgTask's success signal to emit the repository/
+    // files changed signals according to the variant's type:
+    // String -> repository, StringList -> files
+    void changed(const QVariant&);
+
+private:
+    void updateActions(VcsBase::VcsBasePluginPrivate::ActionState) final;
+    bool submitEditorAboutToClose() final;
+
+    // File menu action slots
+    void addCurrentFile();
+    void annotateCurrentFile();
+    void diffCurrentFile();
+    void logCurrentFile();
+    void revertCurrentFile();
+    void statusCurrentFile();
+
+    // Directory menu action slots
+    void diffRepository();
+    void logRepository();
+    void revertMulti();
+    void statusMulti();
+
+    // Repository menu action slots
+    void pull();
+    void push();
+    void update();
+    void import();
+    void incoming();
+    void outgoing();
+    void commit();
+    void showCommitWidget(const QList<VcsBase::VcsBaseClient::StatusItem> &status);
+    void commitFromEditor() override;
+    void diffFromEditorSelected(const QStringList &files);
+
+    void createMenu(const Core::Context &context);
+    void createFileActions(const Core::Context &context);
+    void createDirectoryActions(const Core::Context &context);
+    void createRepositoryActions(const Core::Context &context);
+
+    void describe(const QString &source, const QString &id) { m_client.view(source, id); }
+
+    // Variables
+    MercurialSettings m_settings;
+    MercurialClient m_client{&m_settings};
+
+    OptionsPage m_optionsPage{[this] { configurationChanged(); }, &m_settings};
+
+    Core::CommandLocator *m_commandLocator = nullptr;
+    Core::ActionContainer *m_mercurialContainer = nullptr;
+
+    QList<QAction *> m_repositoryActionList;
+
+    // Menu items (file actions)
+    ParameterAction *m_addAction = nullptr;
+    ParameterAction *m_deleteAction = nullptr;
+    ParameterAction *annotateFile = nullptr;
+    ParameterAction *diffFile = nullptr;
+    ParameterAction *logFile = nullptr;
+    ParameterAction *revertFile = nullptr;
+    ParameterAction *statusFile = nullptr;
+
+    QAction *m_createRepositoryAction = nullptr;
+    QAction *m_menuAction = nullptr;
+
+    QString m_submitRepository;
+
+    bool m_submitActionTriggered = false;
+
+public:
+    VcsSubmitEditorFactory submitEditorFactory {
+        submitEditorParameters,
+        [] { return new CommitEditor; },
+        this
+    };
+
+    VcsEditorFactory logEditorFactory {
+        &logEditorParameters,
+        [this] { return new MercurialEditorWidget(&m_client); },
+        std::bind(&MercurialPluginPrivate::describe, this, _1, _2)
+    };
+
+    VcsEditorFactory annotateEditorFactory {
+        &annotateEditorParameters,
+        [this] { return new MercurialEditorWidget(&m_client); },
+        std::bind(&MercurialPluginPrivate::describe, this, _1, _2)
+    };
+
+    VcsEditorFactory diffEditorFactory {
+        &diffEditorParameters,
+        [this] { return new MercurialEditorWidget(&m_client); },
+        std::bind(&MercurialPluginPrivate::describe, this, _1, _2)
+    };
+};
+
+static MercurialPluginPrivate *dd = nullptr;
 
 MercurialPlugin::~MercurialPlugin()
 {
-    if (m_client) {
-        delete m_client;
-        m_client = nullptr;
-    }
-
-    m_instance = nullptr;
+    delete dd;
+    dd = nullptr;
 }
 
 bool MercurialPlugin::initialize(const QStringList & /* arguments */, QString * /*errorMessage */)
 {
-    Core::Context context(Constants::MERCURIAL_CONTEXT);
-
-    m_client = new MercurialClient;
-    initializeVcs(new MercurialControl(m_client), context);
-
-    addAutoReleasedObject(new OptionsPage(versionControl()));
-
-    connect(m_client, &VcsBaseClient::changed,
-            static_cast<MercurialControl *>(versionControl()), &MercurialControl::changed);
-    connect(m_client, &MercurialClient::needUpdate, this, &MercurialPlugin::update);
-
-    const auto describeFunc = [this](const QString &source, const QString &id) {
-        m_client->view(source, id);
-    };
-    const int editorCount = sizeof(editorParameters)/sizeof(editorParameters[0]);
-    const auto widgetCreator = []() { return new MercurialEditorWidget; };
-    for (int i = 0; i < editorCount; i++)
-        addAutoReleasedObject(new VcsEditorFactory(editorParameters + i, widgetCreator, describeFunc));
-
-    addAutoReleasedObject(new VcsSubmitEditorFactory(&submitEditorParameters,
-        []() { return new CommitEditor(&submitEditorParameters); }));
-
-    const QString prefix = QLatin1String("hg");
-    m_commandLocator = new Core::CommandLocator("Mercurial", prefix, prefix);
-    addAutoReleasedObject(m_commandLocator);
-
-    createMenu(context);
-
-    createSubmitEditorActions();
-
+    dd = new MercurialPluginPrivate;
     return true;
 }
 
-void MercurialPlugin::createMenu(const Core::Context &context)
+void MercurialPlugin::extensionsInitialized()
+{
+    dd->extensionsInitialized();
+}
+
+MercurialPluginPrivate::MercurialPluginPrivate()
+    : VcsBase::VcsBasePluginPrivate(Core::Context(Constants::MERCURIAL_CONTEXT))
+{
+    dd = this;
+
+    setTopicCache(new MercurialTopicCache(&m_client));
+
+    Core::Context context(Constants::MERCURIAL_CONTEXT);
+
+    connect(&m_client, &VcsBaseClient::changed, this, &MercurialPluginPrivate::changed);
+    connect(&m_client, &MercurialClient::needUpdate, this, &MercurialPluginPrivate::update);
+
+    const QString prefix = QLatin1String("hg");
+    m_commandLocator = new Core::CommandLocator("Mercurial", prefix, prefix, this);
+
+    createMenu(context);
+}
+
+void MercurialPluginPrivate::createMenu(const Core::Context &context)
 {
     // Create menu item for Mercurial
     m_mercurialContainer = Core::ActionManager::createMenu("Mercurial.MercurialMenu");
@@ -172,38 +305,38 @@ void MercurialPlugin::createMenu(const Core::Context &context)
     m_menuAction = m_mercurialContainer->menu()->menuAction();
 }
 
-void MercurialPlugin::createFileActions(const Core::Context &context)
+void MercurialPluginPrivate::createFileActions(const Core::Context &context)
 {
     Core::Command *command;
 
     annotateFile = new ParameterAction(tr("Annotate Current File"), tr("Annotate \"%1\""), ParameterAction::EnabledWithParameter, this);
     command = Core::ActionManager::registerAction(annotateFile, Core::Id(Constants::ANNOTATE), context);
     command->setAttribute(Core::Command::CA_UpdateText);
-    connect(annotateFile, &QAction::triggered, this, &MercurialPlugin::annotateCurrentFile);
+    connect(annotateFile, &QAction::triggered, this, &MercurialPluginPrivate::annotateCurrentFile);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     diffFile = new ParameterAction(tr("Diff Current File"), tr("Diff \"%1\""), ParameterAction::EnabledWithParameter, this);
     command = Core::ActionManager::registerAction(diffFile, Core::Id(Constants::DIFF), context);
     command->setAttribute(Core::Command::CA_UpdateText);
-    command->setDefaultKeySequence(QKeySequence(Core::UseMacShortcuts ? tr("Meta+H,Meta+D") : tr("Alt+G,Alt+D")));
-    connect(diffFile, &QAction::triggered, this, &MercurialPlugin::diffCurrentFile);
+    command->setDefaultKeySequence(QKeySequence(Core::useMacShortcuts ? tr("Meta+H,Meta+D") : tr("Alt+G,Alt+D")));
+    connect(diffFile, &QAction::triggered, this, &MercurialPluginPrivate::diffCurrentFile);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     logFile = new ParameterAction(tr("Log Current File"), tr("Log \"%1\""), ParameterAction::EnabledWithParameter, this);
     command = Core::ActionManager::registerAction(logFile, Core::Id(Constants::LOG), context);
     command->setAttribute(Core::Command::CA_UpdateText);
-    command->setDefaultKeySequence(QKeySequence(Core::UseMacShortcuts ? tr("Meta+H,Meta+L") : tr("Alt+G,Alt+L")));
-    connect(logFile, &QAction::triggered, this, &MercurialPlugin::logCurrentFile);
+    command->setDefaultKeySequence(QKeySequence(Core::useMacShortcuts ? tr("Meta+H,Meta+L") : tr("Alt+G,Alt+L")));
+    connect(logFile, &QAction::triggered, this, &MercurialPluginPrivate::logCurrentFile);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     statusFile = new ParameterAction(tr("Status Current File"), tr("Status \"%1\""), ParameterAction::EnabledWithParameter, this);
     command = Core::ActionManager::registerAction(statusFile, Core::Id(Constants::STATUS), context);
     command->setAttribute(Core::Command::CA_UpdateText);
-    command->setDefaultKeySequence(QKeySequence(Core::UseMacShortcuts ? tr("Meta+H,Meta+S") : tr("Alt+G,Alt+S")));
-    connect(statusFile, &QAction::triggered, this, &MercurialPlugin::statusCurrentFile);
+    command->setDefaultKeySequence(QKeySequence(Core::useMacShortcuts ? tr("Meta+H,Meta+S") : tr("Alt+G,Alt+S")));
+    connect(statusFile, &QAction::triggered, this, &MercurialPluginPrivate::statusCurrentFile);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
@@ -212,58 +345,58 @@ void MercurialPlugin::createFileActions(const Core::Context &context)
     m_addAction = new ParameterAction(tr("Add"), tr("Add \"%1\""), ParameterAction::EnabledWithParameter, this);
     command = Core::ActionManager::registerAction(m_addAction, Core::Id(Constants::ADD), context);
     command->setAttribute(Core::Command::CA_UpdateText);
-    connect(m_addAction, &QAction::triggered, this, &MercurialPlugin::addCurrentFile);
+    connect(m_addAction, &QAction::triggered, this, &MercurialPluginPrivate::addCurrentFile);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     m_deleteAction = new ParameterAction(tr("Delete..."), tr("Delete \"%1\"..."), ParameterAction::EnabledWithParameter, this);
     command = Core::ActionManager::registerAction(m_deleteAction, Core::Id(Constants::DELETE), context);
     command->setAttribute(Core::Command::CA_UpdateText);
-    connect(m_deleteAction, &QAction::triggered, this, &MercurialPlugin::promptToDeleteCurrentFile);
+    connect(m_deleteAction, &QAction::triggered, this, &MercurialPluginPrivate::promptToDeleteCurrentFile);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     revertFile = new ParameterAction(tr("Revert Current File..."), tr("Revert \"%1\"..."), ParameterAction::EnabledWithParameter, this);
     command = Core::ActionManager::registerAction(revertFile, Core::Id(Constants::REVERT), context);
     command->setAttribute(Core::Command::CA_UpdateText);
-    connect(revertFile, &QAction::triggered, this, &MercurialPlugin::revertCurrentFile);
+    connect(revertFile, &QAction::triggered, this, &MercurialPluginPrivate::revertCurrentFile);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 }
 
-void MercurialPlugin::addCurrentFile()
+void MercurialPluginPrivate::addCurrentFile()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasFile(), return);
-    m_client->synchronousAdd(state.currentFileTopLevel(), state.relativeCurrentFile());
+    m_client.synchronousAdd(state.currentFileTopLevel(), state.relativeCurrentFile());
 }
 
-void MercurialPlugin::annotateCurrentFile()
+void MercurialPluginPrivate::annotateCurrentFile()
 {
     int currentLine = -1;
     if (Core::IEditor *editor = Core::EditorManager::currentEditor())
         currentLine = editor->currentLine();
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasFile(), return);
-    m_client->annotate(state.currentFileTopLevel(), state.relativeCurrentFile(), QString(), currentLine);
+    m_client.annotate(state.currentFileTopLevel(), state.relativeCurrentFile(), QString(), currentLine);
 }
 
-void MercurialPlugin::diffCurrentFile()
+void MercurialPluginPrivate::diffCurrentFile()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasFile(), return);
-    m_client->diff(state.currentFileTopLevel(), QStringList(state.relativeCurrentFile()));
+    m_client.diff(state.currentFileTopLevel(), QStringList(state.relativeCurrentFile()));
 }
 
-void MercurialPlugin::logCurrentFile()
+void MercurialPluginPrivate::logCurrentFile()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasFile(), return);
-    m_client->log(state.currentFileTopLevel(), QStringList(state.relativeCurrentFile()),
-                  QStringList(), true);
+    m_client.log(state.currentFileTopLevel(), QStringList(state.relativeCurrentFile()),
+                 QStringList(), true);
 }
 
-void MercurialPlugin::revertCurrentFile()
+void MercurialPluginPrivate::revertCurrentFile()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasFile(), return);
@@ -271,62 +404,62 @@ void MercurialPlugin::revertCurrentFile()
     RevertDialog reverter(Core::ICore::dialogParent());
     if (reverter.exec() != QDialog::Accepted)
         return;
-    m_client->revertFile(state.currentFileTopLevel(), state.relativeCurrentFile(), reverter.revision());
+    m_client.revertFile(state.currentFileTopLevel(), state.relativeCurrentFile(), reverter.revision());
 }
 
-void MercurialPlugin::statusCurrentFile()
+void MercurialPluginPrivate::statusCurrentFile()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasFile(), return);
-    m_client->status(state.currentFileTopLevel(), state.relativeCurrentFile());
+    m_client.status(state.currentFileTopLevel(), state.relativeCurrentFile());
 }
 
-void MercurialPlugin::createDirectoryActions(const Core::Context &context)
+void MercurialPluginPrivate::createDirectoryActions(const Core::Context &context)
 {
     auto action = new QAction(tr("Diff"), this);
     m_repositoryActionList.append(action);
     Core::Command *command = Core::ActionManager::registerAction(action, Core::Id(Constants::DIFFMULTI), context);
-    connect(action, &QAction::triggered, this, &MercurialPlugin::diffRepository);
+    connect(action, &QAction::triggered, this, &MercurialPluginPrivate::diffRepository);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     action = new QAction(tr("Log"), this);
     m_repositoryActionList.append(action);
     command = Core::ActionManager::registerAction(action, Core::Id(Constants::LOGMULTI), context);
-    connect(action, &QAction::triggered, this, &MercurialPlugin::logRepository);
+    connect(action, &QAction::triggered, this, &MercurialPluginPrivate::logRepository);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     action = new QAction(tr("Revert..."), this);
     m_repositoryActionList.append(action);
     command = Core::ActionManager::registerAction(action, Core::Id(Constants::REVERTMULTI), context);
-    connect(action, &QAction::triggered, this, &MercurialPlugin::revertMulti);
+    connect(action, &QAction::triggered, this, &MercurialPluginPrivate::revertMulti);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     action = new QAction(tr("Status"), this);
     m_repositoryActionList.append(action);
     command = Core::ActionManager::registerAction(action, Core::Id(Constants::STATUSMULTI), context);
-    connect(action, &QAction::triggered, this, &MercurialPlugin::statusMulti);
+    connect(action, &QAction::triggered, this, &MercurialPluginPrivate::statusMulti);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 }
 
-void MercurialPlugin::diffRepository()
+void MercurialPluginPrivate::diffRepository()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
-    m_client->diff(state.topLevel());
+    m_client.diff(state.topLevel());
 }
 
-void MercurialPlugin::logRepository()
+void MercurialPluginPrivate::logRepository()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
-    m_client->log(state.topLevel());
+    m_client.log(state.topLevel());
 }
 
-void MercurialPlugin::revertMulti()
+void MercurialPluginPrivate::revertMulti()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
@@ -334,100 +467,100 @@ void MercurialPlugin::revertMulti()
     RevertDialog reverter(Core::ICore::dialogParent());
     if (reverter.exec() != QDialog::Accepted)
         return;
-    m_client->revertAll(state.topLevel(), reverter.revision());
+    m_client.revertAll(state.topLevel(), reverter.revision());
 }
 
-void MercurialPlugin::statusMulti()
+void MercurialPluginPrivate::statusMulti()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
 
-    m_client->status(state.topLevel());
+    m_client.status(state.topLevel());
 }
 
-void MercurialPlugin::createRepositoryActions(const Core::Context &context)
+void MercurialPluginPrivate::createRepositoryActions(const Core::Context &context)
 {
     auto action = new QAction(tr("Pull..."), this);
     m_repositoryActionList.append(action);
     Core::Command *command = Core::ActionManager::registerAction(action, Core::Id(Constants::PULL), context);
-    connect(action, &QAction::triggered, this, &MercurialPlugin::pull);
+    connect(action, &QAction::triggered, this, &MercurialPluginPrivate::pull);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     action = new QAction(tr("Push..."), this);
     m_repositoryActionList.append(action);
     command = Core::ActionManager::registerAction(action, Core::Id(Constants::PUSH), context);
-    connect(action, &QAction::triggered, this, &MercurialPlugin::push);
+    connect(action, &QAction::triggered, this, &MercurialPluginPrivate::push);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     action = new QAction(tr("Update..."), this);
     m_repositoryActionList.append(action);
     command = Core::ActionManager::registerAction(action, Core::Id(Constants::UPDATE), context);
-    connect(action, &QAction::triggered, this, &MercurialPlugin::update);
+    connect(action, &QAction::triggered, this, &MercurialPluginPrivate::update);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     action = new QAction(tr("Import..."), this);
     m_repositoryActionList.append(action);
     command = Core::ActionManager::registerAction(action, Core::Id(Constants::IMPORT), context);
-    connect(action, &QAction::triggered, this, &MercurialPlugin::import);
+    connect(action, &QAction::triggered, this, &MercurialPluginPrivate::import);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     action = new QAction(tr("Incoming..."), this);
     m_repositoryActionList.append(action);
     command = Core::ActionManager::registerAction(action, Core::Id(Constants::INCOMING), context);
-    connect(action, &QAction::triggered, this, &MercurialPlugin::incoming);
+    connect(action, &QAction::triggered, this, &MercurialPluginPrivate::incoming);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     action = new QAction(tr("Outgoing..."), this);
     m_repositoryActionList.append(action);
     command = Core::ActionManager::registerAction(action, Core::Id(Constants::OUTGOING), context);
-    connect(action, &QAction::triggered, this, &MercurialPlugin::outgoing);
+    connect(action, &QAction::triggered, this, &MercurialPluginPrivate::outgoing);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     action = new QAction(tr("Commit..."), this);
     m_repositoryActionList.append(action);
     command = Core::ActionManager::registerAction(action, Core::Id(Constants::COMMIT), context);
-    command->setDefaultKeySequence(QKeySequence(Core::UseMacShortcuts ? tr("Meta+H,Meta+C") : tr("Alt+G,Alt+C")));
-    connect(action, &QAction::triggered, this, &MercurialPlugin::commit);
+    command->setDefaultKeySequence(QKeySequence(Core::useMacShortcuts ? tr("Meta+H,Meta+C") : tr("Alt+G,Alt+C")));
+    connect(action, &QAction::triggered, this, &MercurialPluginPrivate::commit);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     m_createRepositoryAction = new QAction(tr("Create Repository..."), this);
     command = Core::ActionManager::registerAction(m_createRepositoryAction, Core::Id(Constants::CREATE_REPOSITORY), context);
-    connect(m_createRepositoryAction, &QAction::triggered, this, &MercurialPlugin::createRepository);
+    connect(m_createRepositoryAction, &QAction::triggered, this, &MercurialPluginPrivate::createRepository);
     m_mercurialContainer->addAction(command);
 }
 
-void MercurialPlugin::pull()
+void MercurialPluginPrivate::pull()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
 
-    SrcDestDialog dialog(SrcDestDialog::incoming, Core::ICore::dialogParent());
+    SrcDestDialog dialog(state, SrcDestDialog::incoming, Core::ICore::dialogParent());
     dialog.setWindowTitle(tr("Pull Source"));
     if (dialog.exec() != QDialog::Accepted)
         return;
-    m_client->synchronousPull(dialog.workingDir(), dialog.getRepositoryString());
+    m_client.synchronousPull(dialog.workingDir(), dialog.getRepositoryString());
 }
 
-void MercurialPlugin::push()
+void MercurialPluginPrivate::push()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
 
-    SrcDestDialog dialog(SrcDestDialog::outgoing, Core::ICore::dialogParent());
+    SrcDestDialog dialog(state, SrcDestDialog::outgoing, Core::ICore::dialogParent());
     dialog.setWindowTitle(tr("Push Destination"));
     if (dialog.exec() != QDialog::Accepted)
         return;
-    m_client->synchronousPush(dialog.workingDir(), dialog.getRepositoryString());
+    m_client.synchronousPush(dialog.workingDir(), dialog.getRepositoryString());
 }
 
-void MercurialPlugin::update()
+void MercurialPluginPrivate::update()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
@@ -436,10 +569,10 @@ void MercurialPlugin::update()
     updateDialog.setWindowTitle(tr("Update"));
     if (updateDialog.exec() != QDialog::Accepted)
         return;
-    m_client->update(state.topLevel(), updateDialog.revision());
+    m_client.update(state.topLevel(), updateDialog.revision());
 }
 
-void MercurialPlugin::import()
+void MercurialPluginPrivate::import()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
@@ -452,49 +585,33 @@ void MercurialPlugin::import()
         return;
 
     const QStringList fileNames = importDialog.selectedFiles();
-    m_client->import(state.topLevel(), fileNames);
+    m_client.import(state.topLevel(), fileNames);
 }
 
-void MercurialPlugin::incoming()
+void MercurialPluginPrivate::incoming()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
 
-    SrcDestDialog dialog(SrcDestDialog::incoming, Core::ICore::dialogParent());
+    SrcDestDialog dialog(state, SrcDestDialog::incoming, Core::ICore::dialogParent());
     dialog.setWindowTitle(tr("Incoming Source"));
     if (dialog.exec() != QDialog::Accepted)
         return;
-    m_client->incoming(state.topLevel(), dialog.getRepositoryString());
+    m_client.incoming(state.topLevel(), dialog.getRepositoryString());
 }
 
-void MercurialPlugin::outgoing()
+void MercurialPluginPrivate::outgoing()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
-    m_client->outgoing(state.topLevel());
+    m_client.outgoing(state.topLevel());
 }
 
-void MercurialPlugin::createSubmitEditorActions()
+void MercurialPluginPrivate::commit()
 {
-    Core::Context context(Constants::COMMIT_ID);
+    if (!promptBeforeCommit())
+        return;
 
-    editorCommit = new QAction(VcsBaseSubmitEditor::submitIcon(), tr("Commit"), this);
-    Core::Command *command = Core::ActionManager::registerAction(editorCommit, Core::Id(Constants::COMMIT), context);
-    command->setAttribute(Core::Command::CA_UpdateText);
-    connect(editorCommit, &QAction::triggered, this, &MercurialPlugin::commitFromEditor);
-
-    editorDiff = new QAction(VcsBaseSubmitEditor::diffIcon(), tr("Diff &Selected Files"), this);
-    Core::ActionManager::registerAction(editorDiff, Core::Id(Constants::DIFFEDITOR), context);
-
-    editorUndo = new QAction(tr("&Undo"), this);
-    Core::ActionManager::registerAction(editorUndo, Core::Id(Core::Constants::UNDO), context);
-
-    editorRedo = new QAction(tr("&Redo"), this);
-    Core::ActionManager::registerAction(editorRedo, Core::Id(Core::Constants::REDO), context);
-}
-
-void MercurialPlugin::commit()
-{
     if (raiseSubmitEditor())
         return;
 
@@ -503,14 +620,14 @@ void MercurialPlugin::commit()
 
     m_submitRepository = state.topLevel();
 
-    connect(m_client, &MercurialClient::parsedStatus, this, &MercurialPlugin::showCommitWidget);
-    m_client->emitParsedStatus(m_submitRepository);
+    connect(&m_client, &MercurialClient::parsedStatus, this, &MercurialPluginPrivate::showCommitWidget);
+    m_client.emitParsedStatus(m_submitRepository);
 }
 
-void MercurialPlugin::showCommitWidget(const QList<VcsBaseClient::StatusItem> &status)
+void MercurialPluginPrivate::showCommitWidget(const QList<VcsBaseClient::StatusItem> &status)
 {
     //Once we receive our data release the connection so it can be reused elsewhere
-    disconnect(m_client, &MercurialClient::parsedStatus, this, &MercurialPlugin::showCommitWidget);
+    disconnect(&m_client, &MercurialClient::parsedStatus, this, &MercurialPluginPrivate::showCommitWidget);
 
     if (status.isEmpty()) {
         VcsOutputWindow::appendError(tr("There are no changes to commit."));
@@ -534,30 +651,29 @@ void MercurialPlugin::showCommitWidget(const QList<VcsBaseClient::StatusItem> &s
     }
 
     QTC_ASSERT(qobject_cast<CommitEditor *>(editor), return);
-    CommitEditor *commitEditor = static_cast<CommitEditor *>(editor);
+    auto commitEditor = static_cast<CommitEditor *>(editor);
     setSubmitEditor(commitEditor);
 
-    commitEditor->registerActions(editorUndo, editorRedo, editorCommit, editorDiff);
     connect(commitEditor, &VcsBaseSubmitEditor::diffSelectedFiles,
-            this, &MercurialPlugin::diffFromEditorSelected);
+            this, &MercurialPluginPrivate::diffFromEditorSelected);
     commitEditor->setCheckScriptWorkingDirectory(m_submitRepository);
 
     const QString msg = tr("Commit changes for \"%1\".").
                         arg(QDir::toNativeSeparators(m_submitRepository));
     commitEditor->document()->setPreferredDisplayName(msg);
 
-    QString branch = versionControl()->vcsTopic(m_submitRepository);
+    const QString branch = vcsTopic(m_submitRepository);
     commitEditor->setFields(m_submitRepository, branch,
-                            m_client->settings().stringValue(MercurialSettings::userNameKey),
-                            m_client->settings().stringValue(MercurialSettings::userEmailKey), status);
+                            m_settings.stringValue(MercurialSettings::userNameKey),
+                            m_settings.stringValue(MercurialSettings::userEmailKey), status);
 }
 
-void MercurialPlugin::diffFromEditorSelected(const QStringList &files)
+void MercurialPluginPrivate::diffFromEditorSelected(const QStringList &files)
 {
-    m_client->diff(m_submitRepository, files);
+    m_client.diff(m_submitRepository, files);
 }
 
-void MercurialPlugin::commitFromEditor()
+void MercurialPluginPrivate::commitFromEditor()
 {
     // Close the submit editor
     m_submitActionTriggered = true;
@@ -565,18 +681,15 @@ void MercurialPlugin::commitFromEditor()
     Core::EditorManager::closeDocument(submitEditor()->document());
 }
 
-bool MercurialPlugin::submitEditorAboutToClose()
+bool MercurialPluginPrivate::submitEditorAboutToClose()
 {
-    CommitEditor *commitEditor = qobject_cast<CommitEditor *>(submitEditor());
+    auto commitEditor = qobject_cast<CommitEditor *>(submitEditor());
     QTC_ASSERT(commitEditor, return true);
     Core::IDocument *editorFile = commitEditor->document();
     QTC_ASSERT(editorFile, return true);
 
-    bool dummyPrompt = false;
     const VcsBaseSubmitEditor::PromptSubmitResult response =
-            commitEditor->promptSubmit(tr("Close Commit Editor"), tr("Do you want to commit the changes?"),
-                                       tr("Message check failed. Do you want to proceed?"),
-                                       &dummyPrompt, !m_submitActionTriggered);
+            commitEditor->promptSubmit(this, nullptr, !m_submitActionTriggered);
     m_submitActionTriggered = false;
 
     switch (response) {
@@ -597,13 +710,13 @@ bool MercurialPlugin::submitEditorAboutToClose()
         QStringList extraOptions;
         if (!commitEditor->committerInfo().isEmpty())
             extraOptions << QLatin1String("-u") << commitEditor->committerInfo();
-        m_client->commit(m_submitRepository, files, editorFile->filePath().toString(),
-                         extraOptions);
+        m_client.commit(m_submitRepository, files, editorFile->filePath().toString(),
+                        extraOptions);
     }
     return true;
 }
 
-void MercurialPlugin::updateActions(VcsBasePlugin::ActionState as)
+void MercurialPluginPrivate::updateActions(VcsBasePluginPrivate::ActionState as)
 {
     if (!enableMenuAction(as, m_menuAction)) {
         m_commandLocator->setEnabled(false);
@@ -625,6 +738,139 @@ void MercurialPlugin::updateActions(VcsBasePlugin::ActionState as)
         repoAction->setEnabled(repoEnabled);
 }
 
+QString MercurialPluginPrivate::displayName() const
+{
+    return tr("Mercurial");
+}
+
+Core::Id MercurialPluginPrivate::id() const
+{
+    return {VcsBase::Constants::VCS_ID_MERCURIAL};
+}
+
+bool MercurialPluginPrivate::isVcsFileOrDirectory(const Utils::FilePath &fileName) const
+{
+    return m_client.isVcsDirectory(fileName);
+}
+
+bool MercurialPluginPrivate::managesDirectory(const QString &directory, QString *topLevel) const
+{
+    QFileInfo dir(directory);
+    const QString topLevelFound = m_client.findTopLevelForFile(dir);
+    if (topLevel)
+        *topLevel = topLevelFound;
+    return !topLevelFound.isEmpty();
+}
+
+bool MercurialPluginPrivate::managesFile(const QString &workingDirectory, const QString &fileName) const
+{
+    return m_client.managesFile(workingDirectory, fileName);
+}
+
+bool MercurialPluginPrivate::isConfigured() const
+{
+    const Utils::FilePath binary = m_settings.binaryPath();
+    if (binary.isEmpty())
+        return false;
+    QFileInfo fi = binary.toFileInfo();
+    return fi.exists() && fi.isFile() && fi.isExecutable();
+}
+
+bool MercurialPluginPrivate::supportsOperation(Operation operation) const
+{
+    bool supported = isConfigured();
+    switch (operation) {
+    case Core::IVersionControl::AddOperation:
+    case Core::IVersionControl::DeleteOperation:
+    case Core::IVersionControl::MoveOperation:
+    case Core::IVersionControl::CreateRepositoryOperation:
+    case Core::IVersionControl::AnnotateOperation:
+    case Core::IVersionControl::InitialCheckoutOperation:
+        break;
+    case Core::IVersionControl::SnapshotOperations:
+        supported = false;
+        break;
+    }
+    return supported;
+}
+
+bool MercurialPluginPrivate::vcsOpen(const QString &filename)
+{
+    Q_UNUSED(filename)
+    return true;
+}
+
+bool MercurialPluginPrivate::vcsAdd(const QString &filename)
+{
+    const QFileInfo fi(filename);
+    return m_client.synchronousAdd(fi.absolutePath(), fi.fileName());
+}
+
+bool MercurialPluginPrivate::vcsDelete(const QString &filename)
+{
+    const QFileInfo fi(filename);
+    return m_client.synchronousRemove(fi.absolutePath(), fi.fileName());
+}
+
+bool MercurialPluginPrivate::vcsMove(const QString &from, const QString &to)
+{
+    const QFileInfo fromInfo(from);
+    const QFileInfo toInfo(to);
+    return m_client.synchronousMove(fromInfo.absolutePath(),
+                                            fromInfo.absoluteFilePath(),
+                                            toInfo.absoluteFilePath());
+}
+
+bool MercurialPluginPrivate::vcsCreateRepository(const QString &directory)
+{
+    return m_client.synchronousCreateRepository(directory);
+}
+
+bool MercurialPluginPrivate::vcsAnnotate(const QString &file, int line)
+{
+    const QFileInfo fi(file);
+    m_client.annotate(fi.absolutePath(), fi.fileName(), QString(), line);
+    return true;
+}
+
+Core::ShellCommand *MercurialPluginPrivate::createInitialCheckoutCommand(const QString &url,
+                                                                   const Utils::FilePath &baseDirectory,
+                                                                   const QString &localName,
+                                                                   const QStringList &extraArgs)
+{
+    QStringList args;
+    args << QLatin1String("clone") << extraArgs << url << localName;
+    auto command = new VcsBase::VcsCommand(baseDirectory.toString(),
+                                           m_client.processEnvironment());
+    command->addJob({m_settings.binaryPath(), args}, -1);
+    return command;
+}
+
+bool MercurialPluginPrivate::sccManaged(const QString &filename)
+{
+    const QFileInfo fi(filename);
+    QString topLevel;
+    const bool managed = managesDirectory(fi.absolutePath(), &topLevel);
+    if (!managed || topLevel.isEmpty())
+        return false;
+    const QDir topLevelDir(topLevel);
+    return m_client.manifestSync(topLevel, topLevelDir.relativeFilePath(filename));
+}
+
+void MercurialPluginPrivate::changed(const QVariant &v)
+{
+    switch (v.type()) {
+    case QVariant::String:
+        emit repositoryChanged(v.toString());
+        break;
+    case QVariant::StringList:
+        emit filesChanged(v.toStringList());
+        break;
+    default:
+        break;
+    }
+}
+
 #ifdef WITH_TESTS
 
 void MercurialPlugin::testDiffFileResolving_data()
@@ -656,7 +902,7 @@ void MercurialPlugin::testDiffFileResolving_data()
 
 void MercurialPlugin::testDiffFileResolving()
 {
-    VcsBaseEditorWidget::testDiffFileResolving(editorParameters[2].id);
+    VcsBaseEditorWidget::testDiffFileResolving(dd->diffEditorFactory);
 }
 
 void MercurialPlugin::testLogResolving()
@@ -676,7 +922,7 @@ void MercurialPlugin::testLogResolving()
                 "date:        Sat Jan 19 04:08:16 2013 +0100\n"
                 "summary:     test-rebase: add another test for rebase with multiple roots\n"
                 );
-    VcsBaseEditorWidget::testLogResolving(editorParameters[0].id, data, "18473:692cbda1eb50", "18472:37100f30590f");
+    VcsBaseEditorWidget::testLogResolving(dd->logEditorFactory, data, "18473:692cbda1eb50", "18472:37100f30590f");
 }
 #endif
 
